var topic = "<!DOCTYPE html>\n\
\n\
<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\">\n\
<script type=\"text/javascript\" src=\"../scripts/prettify.js\"></script><script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"></script>\n\
      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n\
      <meta name=\"product\" content=\"MAYAUL\">\n\
      <meta name=\"release\" content=\"2017\">\n\
      <meta name=\"book\" content=\"Developer\">\n\
      <meta name=\"created\" content=\"2017-05-12\">\n\
      <meta name=\"topicid\" content=\"GUID-3435C604-A3F2-40EC-A50E-BE3018985F41\">\n\
      <meta name=\"topic-type\" content=\"concept\">\n\
      <title>Threading Challenges</title></head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><script language=\"javascript\">var index = \'index.html\';</script><script>$(document).ready(function() { yepnope.injectJs(\"./scripts/ac_common.js\"); });</script><script>$(document).ready(function() { yepnope.injectJs(\"./scripts/multireflink.js\"); });</script><script>$(document).ready(function () { prettyPrint(); } );</script><script>$(\"div#WidgetFloaterPanels,link[href*=\'microsofttranslator.com\'],script[src*=\'microsofttranslator.com\'],script[src*=\'bing.com\']\").remove();</script><script type=\'text/javascript\'>$(\"div#navigation,div#breadcrumbs,div#banner\").attr(\"translate\",\"no\"); var mtLocation = ((location && location.href && location.href.indexOf(\'https\') == 0)?\'https://ssl.microsofttranslator.com\':\'http://www.microsofttranslator.com\')+\'/ajax/v3/WidgetV3.ashx?ctf=True&ui=true&settings=Manual&from=en&hidelanguages=\'; yepnope.injectJs(mtLocation, function() {}, { charset:\'utf-8\', type:\'text/javascript\' } );</script><script></script><script></script><!-- begin MT -->\n\
            \n\
            <div id=\'MicrosoftTranslatorWidget\' class=\'Dark\' style=\'float:right;z-index:100;color:white;background-color:#bbbbbb;height:58px;overflow:hidden\'></div><div id=\"reflinkdiv\"></div>\n\
      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-3435C604-A3F2-40EC-A50E-BE3018985F41\"></a><div class=\"head\">\n\
            <h1>Threading Challenges</h1>\n\
         </div>\n\
         <div class=\"section\"><a name=\"WS73099CC142F487557109353211EEADCAF73-70E6\"></a> \n\
            <h2><a name=\"GUID-6A7EDB1F-2A01-4304-AD36-FE57D519D337\"></a>Correctness\n\
            </h2> \n\
            <p><a name=\"WS73099CC142F487557109353211EEADCAF73-70E5\"></a>One of the most common threading problems is the <em class=\"strong\">race condition</em>, where two threads attempt to modify the same data. Modifying a data element is not\n\
               usually an atomic operation, meaning it is possible for both threads to read the value,\n\
               then update it, then write it. In such cases one of the edits will be overwritten\n\
               by the other. Race conditions can be extremely hard to detect as they may happen only\n\
               very rarely, and be hard to reproduce. Tools such as Intel\'s <em class=\"strong\">ThreadChecker</em> are good for catching race conditions, both ones that actually happen and those that\n\
               could potentially happen.\n\
            </p> \n\
            <p><a name=\"WS73099CC142F487557109353211EEADCAF73-70E4\"></a>Race conditions obviously lead to incorrect results. There are other more subtle problems\n\
               that can occur in threaded code. An example is precision differences in floating point\n\
               arithmetic. With reduction operations, where values are accumulated in different numbers\n\
               of threads and then summed, the number of threads can affect the final result since\n\
               the roundoffs are different:\n\
            </p> \n\
            <div class=\"codeBlock\"><pre class=\"prettyprint\">\n\
#pragma omp parallel for reduction(+: sum)\n\
for ( int i = 0 ; i &lt; n ; i++ ) sum += x[i]*y[i];\n\
</pre></div> \n\
            <p><a name=\"WS73099CC142F487557109353211EEADCAF73-70E3\"></a>This can be a significant effect for simulations, where small differences can accumulate\n\
               over time. Below are two images from an initial threaded implementation of the Maya\n\
               fluids solver that in one place called code similar to the above. The only difference\n\
               between the runs is the number of cores on the system, and so the number of threads\n\
               being used in the solver. Although neither result is <em class=\"mild\">incorrect</em>, the difference in appearance due to differences in roundoff is quite dramatic:\n\
            </p> <a name=\"WS73099CC142F487557109353211EEADCAF73-70E2\"></a><div class=\"figure-anchor\"><img src=\"images/GUID-9115298F-7C0D-45C0-8778-1FC5D09AE897-low.png\"></div> <a name=\"WS73099CC142F487557109353211EEADCAF73-70E1\"></a><div class=\"figure-anchor\"><img src=\"images/GUID-EAD8B7AA-4A3B-47D9-9CCC-1D80DBA90964-low.png\"></div> \n\
            <p><a name=\"WS73099CC142F487557109353211EEADCAF73-70E0\"></a>There are a couple of ways to avoid this. One is to temporarily use higher precision,\n\
               for example, going from floats to doubles during the threaded computation. The idea\n\
               is that the threading benefit is greater than the performance hit of higher precision\n\
               computations. Another way is to break the data into a number of chunks that is fixed\n\
               at compile time, and allocate those chunks to threads at runtime, then accumulate\n\
               the sums in a consistent order. This ensures the size of a chunk never changes, so\n\
               the final sum should always be identical.\n\
            </p> \n\
            <p><a name=\"WS73099CC142F487557109353211EEADCAF73-70DF\"></a>Some C/C++ library functions are not threadsafe as they maintain internal state. Examples\n\
               are STL containers (even reads of STL containers may not be threadsafe in some cases),\n\
               C functions like <span class=\"code\" translate=\"no\">strtok()</span>, and many built-in random number generators. Care must be taken if any of these are\n\
               called from threaded code. TBB provides threadsafe containers as an alternative to\n\
               STL, and there are threadsafe implementations of many C functions which are identified\n\
               with an _r suffix (for reentrant) on Linux and OSX, or an _s suffix on Windows, for\n\
               example, <span class=\"code\" translate=\"no\">strtok_r()</span> is a threadsafe version of <span class=\"code\" translate=\"no\">strtok()</span>, <span class=\"code\" translate=\"no\">strtok_s()</span> is an equivalent threadsafe function on Windows. Unfortunately there are no cross-platform\n\
               standards for these threadsafe functions at the moment.\n\
            </p> \n\
         </div>\n\
         <div class=\"section\"><a name=\"WS73099CC142F487557109353211EEADCAF73-70DE\"></a> \n\
            <h2><a name=\"GUID-217BED7B-1AEF-4018-BFC1-ECBAD34654A8\"></a>Scalability\n\
            </h2> \n\
            <div class=\"section\"><a name=\"WS73099CC142F487557109353211EEADCAF73-70DD\"></a> \n\
               <h3><a name=\"GUID-EABD7113-631E-45F5-ACF7-D2DDC10D4DD7\"></a>Amdahl vs Gustafson\n\
               </h3> \n\
               <p><a name=\"WS73099CC142F487557109353211EEADCAF73-70DC\"></a> <em class=\"strong\">Amdahl\'s law</em> is an observation that for a fixed problem size, performance scaling has diminishing\n\
                  returns as thread counts increase because the serial portion of the code eventually\n\
                  dominates, leading to a ceiling on scalability.\n\
               </p> \n\
               <p><a name=\"WS73099CC142F487557109353211EEADCAF73-70DB\"></a> <em class=\"strong\">Gustafson\'s observation</em> is that usually increasing thread counts are used to address larger problems, so\n\
                  rather than a fixed problem size it is better to think in terms of a fixed runtime.\n\
                  In such cases, scaling can continue to increase indefinitely provided that the fraction\n\
                  of work in parallel versus serial code increases with the problem size.\n\
               </p> \n\
               <p><a name=\"WS73099CC142F487557109353211EEADCAF73-70DA\"></a>For applications like Maya, we are closer to the second case than the first, so there\n\
                  is hope for increasing scalability over time for well written algorithms. Below is\n\
                  an example of the fluids solver showing increasing problem size leads to improved\n\
                  scaling.\n\
               </p> <a name=\"WS73099CC142F487557109353211EEADCAF73-70D9\"></a><div class=\"figure-anchor\"><img src=\"images/GUID-A723E8DB-AC88-46D8-A205-DB88586FC469-low.png\"></div> \n\
            </div> \n\
            <div class=\"section\"><a name=\"WS73099CC142F487557109353211EEADCAF73-70D8\"></a> \n\
               <h3><a name=\"GUID-719F83C5-E488-4193-899E-EA995599546C\"></a>Threading overhead\n\
               </h3> \n\
               <p><a name=\"WS73099CC142F487557109353211EEADCAF73-70D7\"></a>There is an overhead of around 10k cycles to wake up a thread pool. Therefore, any\n\
                  calculation that will take close to this number of cycles to evaluate is not worth\n\
                  threading. For cases where the trip count can vary wildly, and the loop is called\n\
                  many times, it may be worth explicitly providing a lower cutoff to avoid a performance\n\
                  hit. The following diagram shows performance of threaded and unthreaded code with\n\
                  small model sizes for one of Maya\'s deformers. At the smallest sizes threading overhead\n\
                  dominates, and single threaded code is actually faster than multithreaded code. Therefore\n\
                  threading was disabled below the crossover point on this graph.\n\
               </p> <a name=\"WS73099CC142F487557109353211EEADCAF73-70D6\"></a><div class=\"figure-anchor\"><img src=\"images/GUID-E03A2C7E-5BAC-4FE5-8A4C-A67658D8A2C4-low.png\"></div> \n\
               <p><a name=\"WS73099CC142F487557109353211EEADCAF73-70D5\"></a>Another scalability challenge is that locks get increasingly expensive with higher\n\
                  thread counts, so it is worth avoiding locks if at all possible, or at least being\n\
                  sure to re-profile an application when running on a machine with more cores than previously\n\
                  used. An algorithm that scales well to 4 cores may not scale well, or may even slow\n\
                  down, when going to 8 cores.\n\
               </p> \n\
            </div> \n\
            <div class=\"section\"><a name=\"WS73099CC142F487557109353211EEADCAF73-70D4\"></a> \n\
               <h3><a name=\"GUID-5E7E1ED9-4FB8-4F31-903B-FC8E8C724579\"></a>Load balancing\n\
               </h3> \n\
               <p><a name=\"WS73099CC142F487557109353211EEADCAF73-70D3\"></a>Some loops have very different work per iteration. For example a tool that operates\n\
                  on a subset of vertices of a mesh, or a fluids solver working on a volume that has\n\
                  some empty cells, will have some iterations that have little or no work to do. Simply\n\
                  splitting the number of elements equally among threads will be a poor choice for such\n\
                  cases, and it may be worth investigating algorithms and implementations that distribute\n\
                  work more uniformly, for example OpenMP\'s guided or dynamic scheduling, or TBB.\n\
               </p> \n\
            </div> \n\
            <div class=\"section\"><a name=\"WS73099CC142F487557109353211EEADCAF73-70D2\"></a> \n\
               <h3><a name=\"GUID-1A054204-A034-4340-BF6F-67CEDC8E86ED\"></a>False sharing\n\
               </h3> \n\
               <p><a name=\"WS73099CC142F487557109353211EEADCAF73-70D1\"></a>Processor caches work by reading requested data into local caches. However it is not\n\
                  just the actual data that is loaded, but an entire cache line of data around the data\n\
                  item being read. A cache line is usually 64 bytes in size, but it is not guaranteed\n\
                  to be this value.\n\
               </p> \n\
               <p><a name=\"WS73099CC142F487557109353211EEADCAF73-70D0\"></a>False sharing is a situation where multiple threads are reading from, and writing\n\
                  to, variables that are situated close enough in memory that they would reside on the\n\
                  same cache line when read into the cache. When one thread writes to one such variable,\n\
                  the entire cache line is read into the processor cache, and when a different thread\n\
                  attempts to modify the other data element the data must be written from the first\n\
                  processor cache out to a lower level cache or to main memory and then loaded into\n\
                  the other processor cache. This can lead to a ping-pong situation, where a chunk of\n\
                  data is continually bouncing between processor cores, and can clearly cause a significant\n\
                  performance hit.\n\
               </p> \n\
               <p><a name=\"WS73099CC142F487557109353211EEADCAF73-70CF\"></a>The plug-in <span class=\"code\" translate=\"no\">threadedBoundingBox</span>,supplied with Maya, illustrates the cost of false sharing and shows how to avoid\n\
                  it. In this plug-in multiple threads are used to compute one element of a bounding\n\
                  box. Each thread computes a float value representing the minimum X value of the bounding\n\
                  box for a subset of the points. The computed value is written into an array. The computation\n\
                  is run in different ways, single threaded and then multithreaded varying the spacing\n\
                  of the array elements used to hold the output results:\n\
               </p> <a name=\"WS73099CC142F48755-1C09418211F25B18DC4-6309\"></a><div class=\"figure-anchor\"><img src=\"images/GUID-BD51DABD-5C9C-4F08-BB73-EB36F5962AB1-low.png\"></div> \n\
               <p><a name=\"WS73099CC142F487557109353211EEADCAF73-70CD\"></a>In the first test case, each thread writes to an adjacent array element. On a machine\n\
                  with up to 8 cores running separate threads, all threads are likely to write to the\n\
                  same cache line, triggering severe false sharing, since each element is only 4 bytes\n\
                  in size. In such cases performance is actually significantly worse than the unthreaded\n\
                  implementation.\n\
               </p> \n\
               <p><a name=\"WS73099CC142F487557109353211EEADCAF73-70CC\"></a>In the second test case padding is added between active array elements to ensure each\n\
                  value will always be on a separate cache line. In this case, performance is much better.\n\
                  The padding is computed by first querying the cache line size for the current processor.\n\
                  In Maya, there is a new API method called <span class=\"code\" translate=\"no\"><a href=\"javascript:void(0)\" data-symbol=\"MThreadUtils::getCacheLineSize()\" class=\"a_multireflink\" data-reflinkdata=\"[{&quot;path&quot;:&quot;cpp_ref/class_m_thread_utils.html#aa1dfdf0913670708cae93b9b20aa4b11&quot;,&quot;title&quot;:&quot;C++ API Reference&quot;,&quot;relpathtoroot&quot;:&quot;../&quot;}]\">MThreadUtils::getCacheLineSize()</a></span> that will return the correct value at runtime for the host processor. Using this\n\
                  method is the safest way to ensure the code is well behaved for cache line access\n\
                  and also remains well behaved in future with processors that may have larger cache\n\
                  line sizes.\n\
               </p> \n\
            </div> \n\
            <div class=\"section\"><a name=\"WS73099CC142F487557109353211EEADCAF73-70CB\"></a> \n\
               <h3><a name=\"GUID-4E843C45-E883-4EFF-B1A2-939AE1DDE95B\"></a>Hyperthreading\n\
               </h3> \n\
               <p><a name=\"WS73099CC142F487557109353211EEADCAF73-70CA\"></a>Processors such as the Core i7 (Nehalem) have hyperthreading, where a single physical\n\
                  core is treated as two logical cores and two threads of execution can be run on the\n\
                  same physical core. This can provide additional performance but will not provide the\n\
                  same benefit as two physical cores since many core resources must be shared by the\n\
                  threads. In some cases it may lead to poorer performance than a single thread. \n\
               </p> \n\
               <p><a name=\"WS73099CC142F487557109353211EEADCAF73-70C9\"></a>By default, threading APIs like OpenMP and TBB will create one thread per logical\n\
                  core, so will make use of hyperthreading. It is important to test the effect of hyperthreading\n\
                  on your code. If you find it is hurting performance and you choose to run fewer threads\n\
                  than there are logical processors, you may need to check the operating system scheduling\n\
                  behavior. Older operating systems like Windows XP do not necessarily schedule tasks\n\
                  to threads in the optimal way when hyperthreading is present, and sometimes two threads\n\
                  can be running on one core while another core is idle. In such cases it may be best\n\
                  to disable hyperthreading in the system BIOS. \n\
               </p> \n\
            </div> \n\
         </div>\n\
         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n\
   </div></body>\n\
</html>";